library(MASS)
library(nlme)
library(mvtnorm)

sigma<- function(e){
    cluster.means <- NULL
#    cluster.mean <- rep(NA,S*K)
    
    for (i in 1:(S*K)){ 
#        cluster.mean[i] <- mean(e[clusters==i])
        cluster.means <- c(cluster.means,rep(mean(e[clusters==i]),cluster.sizes[i]))
    }
    
    grand.mean <- rep(mean(e),N)    
    
    SSE <- sum((e-cluster.means)^2)
    SST <- sum((e-grand.mean)^2)
    SSA <- SST-SSE

#    SSA1 <- sum(t(nsk)*(cluster.mean-mean(e))^2)

    n.star <- (N-sum(nsk^2)/N)/(S*K-1)

    s.z.hat <- SSE/(N-S*K)
    s.g.hat <- max((SSA-(S*K-1)*s.z.hat)/(N-sum(nsk^2)/N),0)
    return(list(s.z.hat=s.z.hat,s.g.hat=s.g.hat))
}


pv <- function(tau0,tau1,s.z.hat,s.g.hat){
    sig <- s.z.hat+s.g.hat
    Sig <- matrix(c(sig,s.g.hat,s.g.hat,sig),2,2)
    Sigp <- matrix(c(2*sig,s.g.hat+sig,s.g.hat+sig,2*sig),2,2)


    c <- 1-pnorm(0,mean=tau1-tau0,sd=sqrt(2*sig))

    p <- pmvnorm(lower=c(0,0),mean=rep(tau1-tau0,2),sigma=Sigp)[1]

    q <- pmvnorm(lower=c(0,0),mean=rep(tau1-tau0,2),sigma=2*Sig)[1]
 
    return(list(c=c,p=p,q=q)) 
       
}
 

h_r <- function(x,b){min(max(x,-b),b)}

beta_r <- function(b){integrate(f=function(x){h_r(x,b)^2*dnorm(x)}, lower=-Inf, upper=Inf)$value}

beta1_r <- function(b){pchisq(b^2)}

sigma_r <- function(e){

#### Initializing
    
    b.g <- 1.34
    b.z <- 1.34
    mu.t <- mean(e)
    s.g.t <- sigma(e)$s.g.hat
    s.z.t <- sigma(e)$s.z.hat
    ita.t <- rep(NA,S*K)
    a.t <- rep(NA,S*K)
    
##### Pseudo-observation 
    y <- rep(NA,length(e))
    for (i in 1:(S*K)){
        ita.t[i] <- 1+s.z.t/(cluster.sizes[i]*s.g.t)
        a.t[i] <- (mean(e[clusters==i])-mu.t)/ita.t[i]
    }
    y <- mu.t + rep(a.t,cluster.sizes) +  mapply(h_r,e-mu.t-rep(a.t,cluster.sizes),b.z*sqrt(s.z.t))
    O.hat <- a.t-mapply(h_r,a.t,b.g*sqrt(s.g.t))
    mu.hat <- mean(y)-mean(O.hat)
    a.hat <- rep(NA,S*K)
    for (i in 1:(S*K)){a.hat[i] <- (mean(y[clusters==i])-mean(y)-O.hat[i]+mean(O.hat))/ita.t[i]+O.hat[i]}

##### New estimate
    s.g.hs <- sum(mapply(h_r,a.hat,b.g*sqrt(s.g.t))^2)/(S*K*beta_r(b.g))
    
    h2.store <- mapply(h_r,y-mu.hat-rep(a.hat,cluster.sizes),b.z*sqrt(s.z.t))^2
    h2.means <- rep(NA,S)
    for (i in 1:S){h2.means[i] <- mean(h2.store[clusters==i])}
    
    s.z.hs <- sum(h2.means)/(S*K*beta_r(b.z)) 
    
    return(list(s.z.hat=s.z.hs,s.g.hat=s.g.hs))
}



