library(MASS)
library(nlme)
library(mvtnorm)

h <- function(w,E1){sum(w*E1)}
dhi <- function(w,i,E1){S <- length(w);E1[i]-E1[S]}


g <- function(w,V){
    sqrt(sum(w^2*V))
}

dgi <- function(w,i,V){
    S <- length(w)
    (w[i]*V[i]-w[S]*V[S])/g(w,V)
}



dfi <- function(w,i,V0,V1,E1,alpha=0.05){
    Z.alpha <- qnorm(alpha)
    (Z.alpha*dgi(w,i,V0)-dhi(w,i,E1))/g(w,V1) - (Z.alpha*g(w,V0)-h(w,E1))*dgi(w,i,V1)/g(w,V1)^2
}



dfix <- function(w,x,i,V0,V1,E1){
    S <- length(w)
    temp <- w[1:(S-1)]
    temp[i] <- x    
    dfi(c(temp,1-sum(temp)),i,V0,V1,E1)
}




d2fiix <- function(w,x,i,V0,V1,E1){
    S <- length(w)
    temp <- w[1:(S-1)]
    temp[i] <- x    
    d2fii(c(temp,1-sum(temp)),i,V0,V1,E1)
}

####### Coordinate Ascending Method in finding the optimal weights #######

CAopt <- function(V0,E1,V1,tol=1e-9,S=10,maxiter=10000){


    # Start at 1/V0/sum(1/V0)
    iter <- 1/V0/sum(1/V0)
    
    # Maximization
    diff <- 1
    
    itermat <- c(iter,0,diff,f(iter,V0,V1,E1))
    niter <- 1
    
    while (diff >tol && niter < (maxiter+1) ){    
        for (icoor in 1:(S-1)){
            if(iter[icoor]+iter[S]==0){sol <- 0}else{
                if (dfix(iter,0,icoor,V0,V1,E1)*dfix(iter,iter[icoor]+iter[S],icoor,V0,V1,E1)>0){
                    if (d2fiix(iter,iter[icoor],icoor,V0,V1,E1)>0){
                        sol <- uniroot(function(x){dfix(iter,x,icoor,V0,V1,E1)},interval=c(iter[icoor],iter[icoor]+iter[S]),tol=1e-9)$root
                    }else{
                        sol <- uniroot(function(x){dfix(iter,x,icoor,V0,V1,E1)},interval=c(0,iter[icoor]),tol=1e-9)$root
                    }
                }else{
                sol <- uniroot(function(x){dfix(iter,x,icoor,V0,V1,E1)},interval=c(0,iter[icoor]+iter[S]),tol=1e-9)$root
            }
            }
            temp <- iter
            temp[icoor] <- sol
            temp[S] <- 1-sum(temp[1:(S-1)])             
            
            diff <- sqrt(sum((temp-iter)^2))
            if (diff>tol && niter < (maxiter+1)){
                iter <- temp
                itermat <- rbind(itermat,c(iter,icoor,diff,f(iter,V0,V1,E1)))
                niter <- niter+1
            }else{
                iter <- temp
                itermat <- rbind(itermat,c(iter,icoor,diff,f(iter,V0,V1,E1)))
                break
            }
        }
        
    }
    
    
    
    return(list(weights=iter,itermat=itermat))  
}



